{% extends "base.html" %}
{% import "macros.html" as macros with context %}

{% block content %}
<div class="col-md-8 col-md-offset-2">
  <div class="row">
    <a class="pull-right btn btn-default" href="{{ url_for('requests.add_user_note', user_id=user.id) }}">{{ gettext('Add User Note') }}</a>
  </div>
  {% if user.notes %}
    {% for note in user.notes %}
  <blockquote>
    <p>{{ note.content|safe }}</p>
    <footer>{{ gettext('%(noter)s on %(timestamp)s', noter=note.noter.name, timestamp=note.timestamp|datetimeformat) }}</footer>
  </blockquote>
    {% endfor %}
  {% else %}
  <p>{{ gettext("This user doesn't have any notes.") }}</p>
  {% endif %}
</div>
{% endblock content %}
